@@ -173,6 +173,9 @@ cassiusFromString s =
 
 cassiusFile :: FilePath -> Q Exp
 cassiusFile fp = do
+#ifdef GHC_7_4
+    qAddDependentFile fp
+#endif
     contents <- fmap TL.unpack $ qRunIO $ readUtf8File fp
     cassiusFromString contents
 
